/**********************************************************************************
 * Project: TMVA - a Root-integrated toolkit for multivariate data analysis       *
 * Package: TMVA                                                                  *
 * Class  : TMVA_MethodBase                                                       *
 *                                                                                *
 * Description:                                                                   *
 *      Virtual base class for all MVA method                                     *
 *                                                                                *
 * Authors (alphabetical):                                                        *
 *      Andreas Hoecker <Andreas.Hocker@cern.ch> - CERN, Switzerland              *
 *      Xavier Prudent  <prudent@lapp.in2p3.fr>  - LAPP, France                   *
 *      Helge Voss      <Helge.Voss@cern.ch>     - MPI-KP Heidelberg, Germany     *
 *      Kai Voss        <Kai.Voss@cern.ch>       - U. of Victoria, Canada         *
 *                                                                                *
 * Copyright (c) 2005:                                                            *
 *      CERN, Switzerland,                                                        * 
 *      U. of Victoria, Canada,                                                   * 
 *      MPI-KP Heidelberg, Germany,                                               * 
 *      LAPP, Annecy, France                                                      *
 *                                                                                *
 * Redistribution and use in source and binary forms, with or without             *
 * modification, are permitted according to the terms listed in LICENSE           *
 * (http://mva.sourceforge.net/license.txt)                                       *
 *                                                                                *
 * File and Version Information:                                                  *
 * $Id: TMVA_MethodBase.h,v 1.10 2006/04/05 12:35:23 andreas.hoecker Exp $   
 **********************************************************************************/

#ifndef TMVA_MethodBase_H
#define TMVA_MethodBase_H

#include <string>
#include "TObject.h"
#include "TFile.h"
#include "TTree.h"
#include "TString.h"
#include "Riostream.h"
#include "TROOT.h"
#include "TSystem.h"

#include "TMVA_PDF.h"
#include "TMVA_TSpline1.h"
#include "TMVA_Event.h"
#include "TMVA_Types.h"

class TMVA_MethodBase : public TObject {

 public:

  // default constructur
  TMVA_MethodBase( TString jobName,
		  vector<TString>* theVariables, 
		  TTree* theTree = 0, 
		  TString theOption = "", 
		  TDirectory* theBaseDir = 0 );

  // constructor used for Testing + Application of the MVA, only (no training), 
  // using given weight file
  TMVA_MethodBase( vector<TString> *theVariables, 
		   TString weightFile, 
		   TDirectory* theBaseDir = NULL  );

  // default destructur
  virtual ~TMVA_MethodBase( void );

  // training method
  virtual void Train( void ) = 0;

  // write weights to file
  virtual void WriteWeightsToFile( void ) = 0;
  
  // read weights from file
  virtual void ReadWeightsFromFile( void ) = 0;

  // prepare tree branch with the method's discriminating variable
  virtual void PrepareEvaluationTree( TTree* theTestTree );

  // calculate the MVA value
  virtual Double_t mvaValue( TMVA_Event *e ) = 0;

  // evaluate method (resulting discriminating variable) or input varible
  virtual void TestInit(TTree* theTestTree);

  // indivudual initialistion for testing of each method
  // overload this one for individual initialisation of the testing, 
  // it is then called automatically within the global "TestInit" 
  virtual void TestInitLocal(TTree * testTree) {
    return ;
  }

  // test the method
  virtual void Test( TTree * theTestTree );

  // write method specific histos to target file
  virtual void WriteHistosToFile( void ) = 0;

  // accessors
  TString GetMethodName( void ) const         { return m_methodName; }
  TMVA_Types::MVA GetMethod    ( void ) const { return m_method;     }
  TString GetOptions   ( void ) const         { return m_options;    }
  void    SetMethodName( TString methodName ) { m_methodName = methodName; }
  void    AppendToMethodName( TString methodNameSuffix );

  TString GetJobName   ( void ) const         { return m_jobName; }
  void    SetJobName   ( TString jobName )    { m_jobName = jobName; }

  TString GetWeightFileExtension( void ) const            { return m_fileExtension; }
  void    SetWeightFileExtension( TString fileExtension ) { m_fileExtension = fileExtension; } 

  TString GetWeightFileDir( void ) const      { return m_fileDir; }
  void    SetWeightFileDir( TString fileDir );

  vector<TString>*  GetInputVars( void ) const { return m_inputVars; }
  void              SetInputVars( vector<TString>* theInputVars ) { m_inputVars = theInputVars; }

  void     SetWeightFileName( void );
  void     SetWeightFileName( TString );
  TString  GetWeightFileName( void );
  TTree*   GetTrainingTree  ( void ) const { return m_trainingTree; }

  Int_t    GetNvar          ( void ) const { return m_nvar; }

  // variables (and private menber functions) for the Evaluation:
  // get the effiency. It fills a histogram for efficiency/vs/bkg
  // and returns the one value fo the efficiency demanded for 
  // in the TString argument. (Watch the string format)
  virtual Double_t  GetEfficiency   ( TString , TTree *);
  virtual Double_t  GetSignificance ( void );
  virtual Double_t  GetSeparation   ( void );
  virtual Double_t  GetmuTransform  ( TTree * );


  // normalisation init
  virtual void InitNorm( TTree* theTree );

  // normalisation accessors
  Double_t GetXminNorm( Int_t ivar  ) const { return (*m_xminNorm)[ivar]; }
  Double_t GetXmaxNorm( Int_t ivar  ) const { return (*m_xmaxNorm)[ivar]; }
  Double_t GetXminNorm( TString var ) const;
  Double_t GetXmaxNorm( TString var ) const;
  void     SetXminNorm( Int_t ivar,  Double_t x ) { (*m_xminNorm)[ivar] = x; }
  void     SetXmaxNorm( Int_t ivar,  Double_t x ) { (*m_xmaxNorm)[ivar] = x; }
  void     SetXminNorm( TString var, Double_t x );
  void     SetXmaxNorm( TString var, Double_t x );
  void     UpdateNorm ( Int_t ivar,  Double_t x );

  // main normalization method is in TMVA_Tools
  Double_t Norm       ( Int_t ivar,  Double_t x ) const;
  Double_t Norm       ( TString var, Double_t x ) const;
  
  // member functions for the "evaluation" 
  // accessors
  Bool_t   isOK     ( void  )  const { return m_isOK; }

  void WriteHistosToFile( TDirectory* targetDir );

  enum CutOrientation { Negative = -1, Positive = +1 };
  CutOrientation GetCutOrientation() { return m_cutOrientation; }

  enum Type { Signal = 1, Background = 0 };

  Bool_t Verbose( void ) { return m_verbose; }

 public:

  // static pointer to this object
  static TMVA_MethodBase* ThisBase( void ) { return m_ThisBase; }  

 protected:

  void ResetThisBase( void ) { m_ThisBase = this; }

protected:

  TString          m_jobName;
  TString          m_methodName;
  TMVA_Types::MVA  m_method;
  TTree*           m_trainingTree;
  TString          m_testvar;  
  TString          m_testvarPrefix;  
  vector<TString>* m_inputVars;
  TString          m_options;
  TDirectory*      m_baseDir;
  TDirectory*      m_localTDir;

  Bool_t CheckSanity( TTree* theTree = 0 );

  Int_t       m_nvar;

private:

  TString     m_fileExtension;
  TString     m_fileDir;


  TH1*        bookNormTH1( TString, Int_t, Double_t, Double_t, TString );

  TString     m_weightFile;

 protected:

  Bool_t    m_isOK;
  TH1*      m_histS_plotbin;
  TH1*      m_histB_plotbin;
  TH1*      m_histS_highbin;
  TH1*      m_histB_highbin;
  TH1*      m_effS;
  TH1*      m_effB;
  TH1*      m_effBvsS;
  TH1*      m_rejBvsS;
  TH1*      m_histBhatS;
  TH1*      m_histBhatB;
  TH1*      m_histMuS;
  TH1*      m_histMuB;

  // mu-transform
  Double_t  m_x;
  Double_t  m_mode;

  TGraph*   m_graphS;
  TGraph*   m_graphB;
  TGraph*   m_grapheffBvsS;
  TMVA_PDF* m_splS;
  TMVA_PDF* m_splB;
  TSpline*  m_spleffBvsS;
  void      muTransform( void );

 private:

  Double_t  m_meanS;
  Double_t  m_meanB;
  Double_t  m_rmsS; 
  Double_t  m_rmsB;
  Double_t  m_xmin;
  Double_t  m_xmax;

  // verbose flag (debug messages) 
  Bool_t    m_verbose;

 protected:

  Double_t  m_effSatB;
  Int_t     m_nbins;
  Int_t     m_nbinsH;

 private:

  // output types
  Double_t  m_separation;
  Double_t  m_significance;

  // orientation of cut: depends on signal and background mean values
 protected:

  CutOrientation  m_cutOrientation; // +1 if Sig>Bkg, -1 otherwise

  // for root finder
  TMVA_TSpline1*  m_splRefS;
  TMVA_TSpline1*  m_splRefB;

 public:

  // for root finder 
  static Double_t IGetEffForRoot( Double_t );  
  Double_t GetEffForRoot( Double_t );  

 private:

  // normalization
  vector<Double_t>* m_xminNorm;
  vector<Double_t>* m_xmaxNorm;

  // this carrier
  static TMVA_MethodBase* m_ThisBase;

  // Init used in the various constructors
  void Init( void );

  ClassDef(TMVA_MethodBase,0)
};

#endif // TMVA_MethodBase_H

