/**********************************************************************************
 * Project: TMVA - a Root-integrated toolkit for multivariate data analysis       *
 * Package: TMVA                                                                  *
 * Class  : TMVA_MethodCuts                                                       *
 *                                                                                *
 * Description:                                                                   *
 *      Multivariate optimisation of signal efficiency for given background       *
 *      efficiency, using rectangular minimum and maximum requirements on         *
 *      input variables                                                           *
 *                                                                                *
 * Authors (alphabetical):                                                        *
 *      Andreas Hoecker <Andreas.Hocker@cern.ch> - CERN, Switzerland              *
 *      Xavier Prudent  <prudent@lapp.in2p3.fr>  - LAPP, France                   *
 *      Helge Voss      <Helge.Voss@cern.ch>     - MPI-KP Heidelberg, Germany     *
 *      Kai Voss        <Kai.Voss@cern.ch>       - U. of Victoria, Canada         *
 *                                                                                *
 * Copyright (c) 2005:                                                            *
 *      CERN, Switzerland,                                                        * 
 *      U. of Victoria, Canada,                                                   * 
 *      MPI-KP Heidelberg, Germany,                                               * 
 *      LAPP, Annecy, France                                                      *
 *                                                                                *
 * Redistribution and use in source and binary forms, with or without             *
 * modification, are permitted according to the terms listed in LICENSE           *
 * (http://mva.sourceforge.net/license.txt)                                       *
 *                                                                                *
 * File and Version Information:                                                  *
 * $Id: TMVA_MethodCuts.h,v 1.6 2006/04/07 10:35:02 andreas.hoecker Exp $ 
 **********************************************************************************/

#ifndef TMVA_MethodCuts_H
#define TMVA_MethodCuts_H

#include "TMatrix.h"
#include "TMinuit.h"
#include "TRandom.h"
#include "TMVA_MethodBase.h"
#include "TMVA_BinarySearchTree.h"
#include "TMVA_PDF.h"
#include "TMVA_GeneticBase.h"

// static void TMVA_MethodCuts_minimizeFcn( Int_t&, Double_t*, Double_t&, Double_t*, Int_t );

class TMVA_GeneticCuts;

class TMVA_MethodCuts : public TMVA_MethodBase {

 public:

  TMVA_MethodCuts( TString jobName,
		   vector<TString>* theVariables, 
		   TTree* theTree = 0, 
		   TString theOption = "MC:150:10000:",
		   TDirectory* theTargetFile = 0 );
  
  TMVA_MethodCuts( vector<TString> *theVariables, 
		   TString theWeightFile,  
		   TDirectory* theTargetDir = NULL );

  virtual ~TMVA_MethodCuts( void );
    
  // training method
  virtual void Train( void );

  // write weights to file
  virtual void WriteWeightsToFile( void );
  
  // read weights from file
  virtual void ReadWeightsFromFile( void );

  // calculate the MVA value (for CUTs this is just a dummy)
  virtual Double_t mvaValue( TMVA_Event *e );

  // write method specific histos to target file
  virtual void WriteHistosToFile( void );

  // indivudual initialistion of testing of each method test the method
  virtual void TestInitLocal(TTree * testTree);

  // also overwrite:
  virtual Double_t GetSignificance( void )   { return 0; }
  virtual Double_t GetSeparation  ( void )   { return 0; }
  virtual Double_t GetmuTransform ( TTree *) { return 0; }
  virtual Double_t GetEfficiency  ( TString, TTree *);

  // accessors for Minuit 
  static TMinuit* GetTMinuit( void ) { return m_gMinuit; }
  Double_t        ComputeEstimator( Double_t*, Int_t );

  void SetTestSignalEfficiency( Double_t eff ) { m_testSignalEff = eff; }

  // static pointer to this object
  static TMVA_MethodCuts* ThisCuts( void ) { return m_ThisCuts; }  

 protected:

 private:

  enum ConstrainType { ConstrainEffS = 0, 
		       ConstrainEffB } m_constrainType;

  enum FitMethodType { UseEventSelection = 0,
		       UsePDFs,
		       UseMonteCarlo,
		       UseGeneticAlgorithm };

  enum FitParameters { NotEnforced = 0, 
		       StartFromMin,
		       StartFromCenter,
		       StartFromMax,
		       ForceMin,
		       ForceMax,
		       ForceSmart,
		       ForceVerySmart,
		       StartingValuesAreGiven,
		       RandomizeStartingValues };

  enum FitType       { Migrad = 0, Simplex };
  
  // general
  FitMethodType           m_fitMethod;
  FitType                 m_fitType;
  vector<FitParameters>*  m_fitParams;
  Double_t                m_testSignalEff;
  Double_t                m_effSMin;
  Double_t                m_effSMax;

  // for the use of the binary tree method
  TMVA_BinarySearchTree*  m_binaryTreeS;
  TMVA_BinarySearchTree*  m_binaryTreeB;

  // minuit 
  static TMinuit*    m_gMinuit;
  Double_t           m_MinuiteffRef;
  Double_t           m_MinuiteffS;
  Double_t           m_MinuiteffB;
  Double_t           m_MinuitetaMin;
  Double_t           m_Minuiteta0, m_MinuiteffS0, m_MinuiteffB0;  
  Int_t              m_npar;
  Int_t              m_fcnCall;
  vector<Int_t>*     m_rangeSign;
  vector<Double_t>*  m_par0;
  TRandom*           m_trandom;

  // Statistics
  vector<Double_t>*  m_meanS; 
  vector<Double_t>*  m_meanB; 
  vector<Double_t>*  m_rmsS; 
  vector<Double_t>*  m_rmsB; 
  vector<Double_t>*  m_xmin; 
  vector<Double_t>*  m_xmax;    

  TH1*               m_effBvsSLocal;

  // PDF section
  vector<TH1*>*      m_varHistS;
  vector<TH1*>*      m_varHistB;
  vector<TH1*>*      m_varHistS_smooth;
  vector<TH1*>*      m_varHistB_smooth;
  vector<TMVA_PDF*>* m_varPdfS;
  vector<TMVA_PDF*>* m_varPdfB;             
  
  // MC method
  Int_t              m_nRandCuts;
  Double_t**         m_cutMin;
  Double_t**         m_cutMax;

  static TMVA_MethodCuts*   m_ThisCuts;

  void     InitTMinuitAndFit   ( FitParameters fitParam = NotEnforced, 
			         vector<Double_t>* parStart = NULL );
  void     MatchParsToCuts     ( Double_t*, Double_t*, Double_t* );
  void     MatchCutsToPars     ( Double_t*, Double_t*, Double_t* );
  void     Fit                 ( void );
  void     CheckErr            ( TString, Int_t );
  void     Backup              ( Double_t&, Double_t&, Double_t&, vector<Double_t>* );
  void     CreateVariablePDFs  ( void );
  Bool_t   SanityChecks        ( void );
  void     GetEffsfromSelection(  Double_t* cutMin, Double_t* cutMax,
				  Double_t& effS, Double_t& effB);
  void     GetEffsfromPDFs     (  Double_t* cutMin, Double_t* cutMax,
				  Double_t& effS, Double_t& effB );
  void     InitCuts( void );

  ClassDef(TMVA_MethodCuts,0)  
};

typedef std::pair<double,double> LowHigh;

class TMVA_GeneticCuts :  public TMVA_GeneticBase {

 public:
  TMVA_GeneticCuts( int size, vector<LowHigh*> ranges ) 
    : TMVA_GeneticBase( size, ranges ) {}
		
  Double_t fitnessFunction( vector<Double_t> parameters );
		
  Double_t newFitness( Double_t oldF, Double_t newF ) { return oldF + newF; }
};



#endif // TMVA_MethodCuts_H
