/**********************************************************************************
 * Project: TMVA - a Root-integrated toolkit for multivariate data analysis       *
 * Package: TMVA                                                                  *
 * Class  : TMVA_MethodLikelihood                                                 *
 *                                                                                *
 * Description:                                                                   *
 *      Likelihood analysis (also denoted: "parametric approach")                 *
 *      Also implemented is a "diagonalized likelihood approach",                 *
 *      which improves over the uncorrelated likelihood ansatz by transforming    *
 *      linearly the input variables into a diagonal space, using the square-root *
 *      of the covariance matrix. This approach can be chosen by inserting        *
 *      the letter "D" into the option string.                                    *
 *                                                                                *
 * Authors (alphabetical):                                                        *
 *      Andreas Hoecker <Andreas.Hocker@cern.ch> - CERN, Switzerland              *
 *      Xavier Prudent  <prudent@lapp.in2p3.fr>  - LAPP, France                   *
 *      Helge Voss      <Helge.Voss@cern.ch>     - MPI-KP Heidelberg, Germany     *
 *      Kai Voss        <Kai.Voss@cern.ch>       - U. of Victoria, Canada         *
 *                                                                                *
 * Copyright (c) 2005:                                                            *
 *      CERN, Switzerland,                                                        * 
 *      U. of Victoria, Canada,                                                   * 
 *      MPI-KP Heidelberg, Germany,                                               * 
 *      LAPP, Annecy, France                                                      *
 *                                                                                *
 * Redistribution and use in source and binary forms, with or without             *
 * modification, are permitted according to the terms listed in LICENSE           *
 * (http://mva.sourceforge.net/license.txt)                                       *
 *                                                                                *
 * File and Version Information:                                                  *
 * $Id: TMVA_MethodLikelihood.h,v 1.3 2006/03/29 17:15:12 andreas.hoecker Exp $      
 **********************************************************************************/

#ifndef TMVA_MethodLikelihood_H
#define TMVA_MethodLikelihood_H

#include "TH1D.h"
#include "TMatrixD.h"
#include "TMVA_MethodBase.h"
#include "TMVA_PDF.h"

// for convenience...
#define TMVA_MethodLikelihood_max_nvar__ 200


class TMVA_MethodLikelihood : public TMVA_MethodBase {

 public:

  TMVA_MethodLikelihood( TString jobName, 
			vector<TString>* theVariables, 
			TTree* theTree = 0,
			TString theOption = "",
			TDirectory* theTargetDir = 0 );
  
  TMVA_MethodLikelihood( vector<TString> *theVariables, 
			 TString theWeightFile,  
			 TDirectory* theTargetDir = NULL );

  virtual ~TMVA_MethodLikelihood( void );
    
  // training method
  virtual void Train( void );

  // write weights to file
  virtual void WriteWeightsToFile( void );
  
  // read weights from file
  virtual void ReadWeightsFromFile( void );

  // calculate the MVA value
  virtual Double_t mvaValue( TMVA_Event *e );

  // write method specific histos to target file
  virtual void WriteHistosToFile( void ) ;

  // additional accessor
  Bool_t DecorrVarSpace( void ) { return m_decorrVarSpace; }

  // special normalization class for diagonalized Fisher
  // override bas-class function
  //  void InitNorm( TTree* theTree );

 protected:

 private:

  TFile* m_fin;

  TMVA_PDF::SmoothMethod m_smoothMethod;

  // number of events (tot, signal, background)
  Int_t            m_nevt;
  Int_t            m_nsig;
  Int_t            m_nbgd;

  Int_t            m_nsmooth;
  Double_t         m_epsilon;
  TMatrixD*        m_sqS;
  TMatrixD*        m_sqB;

  vector<TH1*>*    m_histSig;
  vector<TH1*>*    m_histBgd; 
  vector<TH1*>*    m_histSig_smooth; 
  vector<TH1*>*    m_histBgd_smooth; 
  
  TList *m_SigPDFHist, *m_BgdPDFHist;

  vector<TMVA_PDF*>*  m_PDFSig;
  vector<TMVA_PDF*>*  m_PDFBgd;

  Int_t     m_nbins;
  Int_t     m_averageEvtPerBin; // average events per bin. Used to calculate nbins

  // diagonalisation of variable space
  Bool_t    m_decorrVarSpace;
  void GetSQRMats( void );
  void InitLik( void );
   
  ClassDef(TMVA_MethodLikelihood,0)  
};

#endif // TMVA_MethodLikelihood_H
