/**********************************************************************************
 * Project: TMVA - a Root-integrated toolkit for multivariate data analysis       *
 * Package: TMVA                                                                  *
 * Classes: TMVA_Node, TMVA_NodeID                                                *
 *                                                                                *
 * Description:                                                                   *
 *      Node for the BinarySearch- or Decision-Trees                              *
 *                                                                                *
 *                                                                                *
 * Authors (alphabetical):                                                        *
 *      Andreas Hoecker <Andreas.Hocker@cern.ch> - CERN, Switzerland              *
 *      Xavier Prudent  <prudent@lapp.in2p3.fr>  - LAPP, France                   *
 *      Helge Voss      <Helge.Voss@cern.ch>     - MPI-KP Heidelberg, Germany     *
 *      Kai Voss        <Kai.Voss@cern.ch>       - U. of Victoria, Canada         *
 *                                                                                *
 * Copyright (c) 2005:                                                            *
 *      CERN, Switzerland,                                                        * 
 *      U. of Victoria, Canada,                                                   * 
 *      MPI-KP Heidelberg, Germany                                                * 
 *      LAPP, Annecy, France                                                      *
 *                                                                                *
 * Redistribution and use in source and binary forms, with or without             *
 * modification, are permitted according to the terms listed in LICENSE           *
 * (http://mva.sourceforge.net/license.txt)                                       *
 *                                                                                *
 * File and Version Information:                                                  *
 * $Id: TMVA_Node.h,v 1.3 2006/03/14 17:05:48 helgevoss Exp $    
 **********************************************************************************/

#ifndef TMVA_Node_H
#define TMVA_Node_H

//#include <iostream>
//#include <fstream>
//#include <vector>
#include "Riostream.h"
//#include "TVector.h"
//#include "TTree.h"
//#include "TString.h"

#include "TMVA_Event.h"

// a class used to identify a Node; (needed for recursive reading from text file)
// (currently it is NOT UNIQUE... but could eventually made it

class TMVA_NodeID {

 public:

  TMVA_NodeID( Int_t d=0, std::string p="Root" ) : m_depth( d ), m_pos( p ) {}
  ~TMVA_NodeID() {}

  void depth(const Int_t d){m_depth=d;}
  Int_t depth() const {return m_depth;}
  void pos(const std::string s) {m_pos=s;}
  std::string pos() const {return m_pos;}

 private:

  Int_t        m_depth;
  std::string  m_pos;
};


// a node in the tree structure
class TMVA_Node {

  friend ostream& operator << (ostream& os, const TMVA_Node& node);
  friend ostream& operator << (ostream& os, const TMVA_Node* node);

public:

  TMVA_Node( TMVA_Event* e =NULL, Bool_t o=kFALSE ) : m_event( e ), m_left( NULL ), 
    m_right( NULL ), m_parent ( NULL ), m_selector( -1 ), m_eventOwnership ( o ) {}

  TMVA_Node( TMVA_Node* p ) : m_event( NULL ), m_left( NULL ), 
    m_right( NULL ), m_parent ( p ), m_selector( -1 ), m_eventOwnership (kFALSE) {}

  virtual ~TMVA_Node ();

  virtual bool goesRight( const TMVA_Event* ) const;
  virtual bool goesLeft ( const TMVA_Event* ) const;
  virtual bool equalsMe ( const TMVA_Event* ) const;

  inline TMVA_Node* left  () const { return m_left;   }
  inline TMVA_Node* right () const { return m_right;  }
  inline TMVA_Node* parent() const { return m_parent; }

  inline void parent(TMVA_Node* p) { m_parent = p;} // Set the parent pointer of the node.
  inline void left  (TMVA_Node* l) { m_left   = l;}     // Set the left pointer of the node.
  inline void right (TMVA_Node* r) { m_right  = r;}   // Set the right pointer of the node.
  
  inline void selector( const Short_t i) { m_selector = i; }
  inline void selector( const Int_t i  ) { m_selector = Short_t(i); }
  inline Short_t selector() const { return m_selector; }
  inline void data( TMVA_Event* e ) { m_event = e; }
  inline TMVA_Event* data() const { return m_event; }

  int  countMeAndAllDaughters() const;
  void print( ostream& os ) const;

  virtual void printRec( ostream& os, const int depth=0, const std::string pos="root" ) const;
  virtual TMVA_NodeID readRec( ifstream& is, TMVA_NodeID nodeID, TMVA_Node* parent=NULL );

  Bool_t      eventOwnership( void           ) { return m_eventOwnership; }
  void        eventOwnership( const Bool_t b ) { m_eventOwnership = b; }

 private:
  TMVA_Event* m_event;

  TMVA_Node*  m_left;    // pointers to the two "daughter" nodes
  TMVA_Node*  m_right;   // pointers to the two "daughter" nodes
  TMVA_Node*  m_parent;  // the previous (parent) node

  Short_t     m_selector;
  Bool_t      m_eventOwnership;
};



#endif // TMVA_Node_H

