/**********************************************************************************
 * Project: TMVA - a Root-integrated toolkit for multivariate data analysis       *
 * Package: TMVA                                                                  *
 * Namespace : TMVA_Utils                                                         *
 *                                                                                *
 * Description:                                                                   *
 *      Global utility routines                                                   *
 *                                                                                *
 * Authors (alphabetical):                                                        *
 *      Andreas Hoecker <Andreas.Hocker@cern.ch> - CERN, Switzerland              *
 *      Xavier Prudent  <prudent@lapp.in2p3.fr>  - LAPP, France                   *
 *      Helge Voss      <Helge.Voss@cern.ch>     - MPI-KP Heidelberg, Germany     *
 *      Kai Voss        <Kai.Voss@cern.ch>       - U. of Victoria, Canada         *
 *                                                                                *
 * Copyright (c) 2005:                                                            *
 *      CERN, Switzerland,                                                        * 
 *      U. of Victoria, Canada,                                                   * 
 *      MPI-KP Heidelberg, Germany,                                               * 
 *      LAPP, Annecy, France                                                      *
 *                                                                                *
 * Redistribution and use in source and binary forms, with or without             *
 * modification, are permitted according to the terms listed in LICENSE           *
 * (http://mva.sourceforge.net/license.txt)                                       *
 *                                                                                *
 * File and Version Information:                                                  *
 * $Id: TMVA_Utils.h,v 1.1 2006/02/19 23:20:49 helgevoss Exp $           
 **********************************************************************************/

#ifndef TMVA_UTILS_H
#define TMVA_UTILS_H 

#include "TObject.h"
#include "TString.h"
#include "Riostream.h"
#include <vector>
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <math.h>
#include <string>
#include <sstream>
#include <vector>
#include <algorithm>


using std::vector;

namespace TMVA_Utils {

  vector<Double_t> MVADiff( vector<Double_t>&, vector<Double_t>& );
  Int_t locateMax( vector<double> );
  Int_t locateMin( vector<double> );  
  void scale     ( vector<Double_t>&, Double_t );
  void scale     ( vector<Float_t> &, Float_t  );
  
  // re-arrange a vector of arrays (vectors) in a way such that the first array
  // is ordered, and the other arrays reshuffeld accordingly
  void usefulSortDescending( vector< vector<Double_t> >&, vector<TString>* vs = 0 );
  void usefulSortAscending ( vector< vector<Double_t> >& );
    
  void usefulSortDescending( vector<Double_t>& );
  void usefulSortAscending ( vector<Double_t>& );

  int  maxElement ( vector<Double_t>& );
  int  minElement ( vector<Double_t>& );

  std::vector<std::string> hvStringSplit(const std::string& str,
					 const std::string delimiter = " ");
  
  std::string hvSkipLeadingWhiteSpace(const std::string &str);
  std::string hvRemoveWhiteSpace(const std::string &str);
  std::string hvRemoveCharacter(const std::string &str,std::string rm );
  
// ============================================================================
//
//   a TMVA_ConfigLine class to be used with my Parser (TMVA_Config)
//   to define "Data Cards" or "Input Paramters" that can be read in
//   in a flexible way  using a config file
// 
//   currently supported data types:
//    
//   int, vector<int>, long, vector<long>, double, vector<double>, std::string
//

//   author: Helge Voss  2005
//
// ============================================================================
// a parameter file could look like: (whitespaece is ignored, // comments, too)

//  SomeParameter   = 6;           // a  parameter
//  SomeArray[4] = { 1,3,5,4 }; // an array of integers
//  AnotherArray[2] = { 1.0, 2.0 };   // an array of doubles
//  aString     = "/dev/lpt0"; // device name (i.e. a "string")
//
// //in order to be define such config lines, you'd have to write:
//
//   TMVA_Config *config=new TMVA_Config();
//
//   config->defineParameter(
//     new TMVA_ConfigLineIntValue("SomeParameter", 23,  "%d",    " give it a description if you want"));
//    
//   config->defineParameter(
//     TMVA_ConfigLine("SomeArray[4]", <your vector with default values>,  "%d",    "e.g. array of integers"));
//
//   config->defineParameter(
//     TMVA_ConfigLine("AnotherArray[2]", < "%5.3f", "array of 2 float)"));
//
//   config->defineParameter(
//     TMVA_ConfigLine("aString",   TYPE_STRING, 32, "", "device name"));
//
//   config->readConfigFile("myParserTest.cfg");
// ============================================================================

class TMVA_ConfigLine{
    
 public:
  TMVA_ConfigLine(std::string name,  
		  std::string format, std::string description);
  virtual ~TMVA_ConfigLine(){}
    
  std::string name(){return theName;}
  virtual void* value()=0;
  virtual void  value(void*)=0;
  virtual std::string type()=0;

  std::string WriteFormat(){return theWriteFormat;}
  std::string Description(){return theDescription;}
  virtual void print()=0;

 private:
  std::string          theName;
  std::string          theWriteFormat;
  std::string          theDescription;
};

class TMVA_ConfigLineIntValue : public TMVA_ConfigLine {    
 public:
  TMVA_ConfigLineIntValue(std::string name, int  defaultValue, 
		       std::string format, std::string description);
  ~TMVA_ConfigLineIntValue(){};
    
  virtual int* value(){return &theValue;};
  virtual void value(void* v){theValue = *((int*)v);};
  virtual std::string type(){return  "int";};
  virtual void print();
 private:
  int theValue;
};

class TMVA_ConfigLineIntVector : public TMVA_ConfigLine {    
 public:
  TMVA_ConfigLineIntVector(std::string name, std::vector<int> defaultValue,
			std::string format, std::string description);
  ~TMVA_ConfigLineIntVector(){};
    
  //  virtual void* value(){return &theValue;};
  virtual std::vector<int>* value(){return &theValue;};
  virtual void value(void* v){theValue = *((std::vector<int>*)(v));};
  virtual std::string type(){return  "intVector";};
  virtual void print();
 private:
  std::vector<int> theValue;
};


class TMVA_ConfigLineLongValue : public TMVA_ConfigLine {    
 public:
  TMVA_ConfigLineLongValue(std::string name, long  defaultValue, 
		       std::string format, std::string description);
  ~TMVA_ConfigLineLongValue(){};
    
  virtual long* value(){return &theValue;};
  virtual void value(void* v){theValue = *((long*)v);};
  virtual std::string type(){return  "long";};
  virtual void print();
 private:
  long theValue;
};

class TMVA_ConfigLineLongVector : public TMVA_ConfigLine {    
 public:
  TMVA_ConfigLineLongVector(std::string name, std::vector<long> defaultValue,
			std::string format, std::string description);
  ~TMVA_ConfigLineLongVector(){};
    
  //  virtual void* value(){return &theValue;};
  virtual std::vector<long>* value(){return &theValue;};
  virtual void value(void* v){theValue = *((std::vector<long>*)(v));};
  virtual std::string type(){return  "longVector";};
  virtual void print();
 private:
  std::vector<long> theValue;
};



class TMVA_ConfigLineDoubleValue : public TMVA_ConfigLine {    
 public:
  TMVA_ConfigLineDoubleValue(std::string name, double defaultValue,
			  std::string format, std::string description);
  ~TMVA_ConfigLineDoubleValue(){};
    
  virtual double* value(){return &theValue;};
  virtual void value(void* v){theValue = *((double*)v);};
  virtual std::string type(){return  "double";};
  virtual void print();
 private:
  double theValue;
};

class TMVA_ConfigLineDoubleVector : public TMVA_ConfigLine {    
 public:
  TMVA_ConfigLineDoubleVector(std::string name, std::vector<double> defaultValue, 
			   std::string format, std::string description);
  virtual ~TMVA_ConfigLineDoubleVector(){};
    
  virtual std::vector<double>* value(){return &theValue;};
  virtual void value(void* v){theValue = *( (std::vector<double>*)v);};
  virtual std::string type(){return  "doubleVector";};
  virtual void print();
 private:
  std::vector<double> theValue;
};

class TMVA_ConfigLineString : public TMVA_ConfigLine {    
 public:
  TMVA_ConfigLineString(std::string name, std::string, 
		     std::string format, std::string description);
  virtual ~TMVA_ConfigLineString(){};
    
  virtual std::string* value(){return &theValue;};
  virtual void value(void* v){theValue = *((std::string*)v);};
  virtual std::string type(){return  "string";};
  virtual void print();
 private:
  std::string theValue;
};

class TMVA_ConfigLineBool : public TMVA_ConfigLine {    
 public:
  TMVA_ConfigLineBool(std::string name, bool,
		     std::string format, std::string description);
  virtual ~TMVA_ConfigLineBool(){}
    
  virtual bool* value(){return &theValue;};
  virtual void value(void* v){theValue = *((bool*)v);};
  virtual std::string type(){return  "bool";};
  virtual void print();
 private:
  bool theValue;
};

// ============================================================================
//
//   Parser (TMVA_Config) to read config lines according to TMVA_ConfigLine.h
//   define "Data Cards" or "Input Paramters" that can be read in
//   in a flexible way  using a config file
// 
//   author: Helge Voss  2005
//
// ============================================================================
// a parameter file could look like: (whitespaece is ignored, // comments, too)
/*

 SomeParameter   = 6;           // a  parameter
 SomeArray[4] = { 1,3,5,4 }; // an array of integers
 AnotherArray[2] = { 1.0, 2.0 };   // an array of doubles
 aString     = "/dev/lpt0"; // device name (i.e. a "string")

//in order to be define such config lines, you'd have to write:

  TMVA_Config *config=new TMVA_Config();

  config->defineParameter(
    TMVA_ConfigLine("SomeParameter", TYPE_INT,     1, "%d",    " give it a description if you want"));
    
  config->defineParameter(
    TMVA_ConfigLine("SomeArray[4]", TYPE_INT_VECTOR,     4, "%d",    "e.g. array of integers"));

  config->defineParameter(
    TMVA_ConfigLine("AnotherArray[2]", TYPE_DOUBLE_VECTOR,  2, "%5.3f", "array of 2 float)"));

  config->defineParameter(
    TMVA_ConfigLine("aString",   TYPE_STRING, 32, "", "device name"));

  config->readConfigFile("myParserTest.cfg");
*/

class TMVA_Config {
public:
    static TMVA_Config* Instance();

    ~TMVA_Config();

    void defineParameter(TMVA_ConfigLine*);
    int readParameterFile(char* FileName);
    int readParameterFile(std::string FileName);

    TMVA_ConfigLine* parameter(std::string ParaName);
    void* value(std::string ParaName);
private:
    TMVA_Config(){};
    static TMVA_Config* theInstance;

    std::vector<TMVA_ConfigLine*> TMVA_ParameterList;
    
 
};


}

#endif //TMVA_UTILS_H

// ============================================================================
